#include "Scene.h"

#include "RootNode.h"
#include "FontsManager.h"
#include "SceneItem.h"

#include <iostream>

Scene::Scene()
    : mRootNode(std::make_unique<RootNode>())
{
    mRootNode->setName("Scene_RootNode");
}

Scene::~Scene()
{

}

void Scene::update(FontsManager* fontsManager)
{
    mSceneItems.clear();
    updateNode(mRootNode.get(), fontsManager);
}

void Scene::addNode(AbstractVisualNode* node)
{
    mRootNode->addChild(node);
}

bool Scene::isEmpty() const
{
    return mRootNode->getNumChildren() == 0;
}

void Scene::updateNode(AbstractVisualNode* node, FontsManager* fontsManager)
{
    for (auto child : node->getChildren())
    {
        if (child->getIsVisible())
        {
            updateNode(child, fontsManager);
        }
    }

    node->update(fontsManager);

    for (std::size_t idx=0; idx< node->getNumSceneItems(); idx++)
    {
        if (auto item = node->getSceneItem(idx))
        {
            mSceneItems.push_back(item);
        }
    }
}

const std::vector<SceneItem*>& Scene::getItems() const
{
    return mSceneItems;
}
